@@ -4,12 +4,12 @@ OpenTmiReport module
 import os
 import time
 import datetime
-import multiprocessing
 import uuid
+import inspect
+from multiprocessing.dummy import Pool as ThreadPool
 # 3rd party modules
-from joblib import Parallel, delayed
 from opentmi_client import OpenTmiClient, Result
-from opentmi_client.api import Dut, File, Provider
+from opentmi_client.api import Dut, File, Provider, Testcase
 # app modules
 from . import __pytest_info__
 
@@ -26,6 +26,8 @@ class OpenTmiReport:
         """
         self.test_logs = []
         self.results = []
+        self.tests = []
+        self._items = dict()
         self.errors = self.failed = 0
         self.passed = self.skipped = 0
         self.xfailed = self.xpassed = 0
@@ -39,6 +41,7 @@ class OpenTmiReport:
         self.rerun = 0 if has_rerun else None
         self.config = config
         host = config.getoption("opentmi")
+        self._store_logs = config.getoption('opentmi_store_logs')
         self._client = OpenTmiClient(host)
 
     def _append_passed(self, report):
@@ -72,11 +75,11 @@ class OpenTmiReport:
     def _append_skipped(self, report):
         result = self._new_result(report)
         result.execution.verdict = 'skip'
-        result.execution.note = report.longrepr[2]
         if hasattr(report, "wasxfail"):
             self.xfailed += 1
-            result.execution.note = 'xskil'
+            result.execution.note = f'wasxfail: {report.longrepr.reprcrash.message}'
         else:
+            result.execution.note = report.longrepr[2]
             self.skipped += 1
         self.results.append(result)
 
@@ -97,14 +100,57 @@ class OpenTmiReport:
         result.execution.note = 'rerun'
         self.results.append(result)
 
+    @staticmethod
+    def _get_tcid(report):
+        return report.head_line.rstrip("[]")
+
+    def _parse_test(self, report):
+        tcid = OpenTmiReport._get_tcid(report)
+        key = OpenTmiReport._get_test_key(report)
+        item = self._items[key]
+        doc = inspect.getdoc(item.obj)
+        test = Testcase(tcid=tcid)
+        test.status.value = "released"
+        if doc:
+            test.other_info.description = doc
+        type_list = ['installation',
+                     'compatibility',
+                     'smoke',
+                     'regression',
+                     'acceptance',
+                     'alpha',
+                     'beta',
+                     'stability',
+                     'functional',
+                     'destructive',
+                     'performance',
+                     'reliability']
+        keywords = list(report.keywords.keys())
+        test.other_info.keywords = keywords
+        for keyword in keywords:
+            if keyword in type_list:
+                test.other_info.type = keyword
+                break
+        if report.skipped:
+            test.execution.skip.value = True
+            if hasattr(report, 'wasxfail'):
+                test.execution.skip.reason = report.wasxfail
+            else:
+                test.execution.skip.reason = report.longrepr[2]
+        return test
+
     # pylint: disable=too-many-statements
     def _new_result(self, report):
-        result = Result(tcid=report.nodeid)
+        tcid = self._get_tcid(report)
+
+        self.tests.append(self._parse_test(report).data)
+
+        result = Result(tcid=tcid)
         result.execution.duration = report.duration
         result.execution.environment.framework.name = __pytest_info__.project_name
         result.execution.environment.framework.version = __pytest_info__.version
         result.execution.sut.commit_id = os.environ.get('GIT_COMMIT', "")
-        result.execution.sut.branch = os.environ.get('GIT_BRANCH ', "")
+        result.execution.sut.branch = os.environ.get('GIT_BRANCH', "")
         result.job.id = os.environ.get('BUILD_TAG', str(uuid.uuid1()))
         result.campaign = os.environ.get('JOB_NAME', "")
         result.execution.profiling = dict()
@@ -115,6 +161,8 @@ class OpenTmiReport:
         if report.keywords:
             result.execution.profiling['keywords'] = []
         for key in report.keywords:
+            if key == "":
+                continue
             result.execution.profiling['keywords'].append(key)
         dut = None
         for item in self.config.option.metadata:
@@ -123,10 +171,14 @@ class OpenTmiReport:
             # Dut
             if key.startswith('DUT') and not dut:
                 dut = Dut()
-                result.append_dut(dut)
+                dut.type = 'hw'
+                result.execution.append_dut(dut)
             if key == 'DUT_SERIAL_NUMBER':
                 dut.serial_number = value
-                dut.type = 'hw'
+            elif key == 'DUT_TYPE':
+                dut.type = value
+            # elif key == 'DUT_PLATFORM':
+            #     dut.platform = value
             elif key == 'DUT_VERSION':
                 dut.ver = value
             elif key == 'DUT_VENDOR':
@@ -144,15 +196,21 @@ class OpenTmiReport:
                 result.execution.sut.append_fut(value)
             elif key == 'SUT_COMMIT_ID':
                 result.execution.sut.commit_id = value
+            elif key == 'SUT_BRANCH':
+                result.execution.sut.branch = value
 
-        if report.capstdout:
+            # push to generic metadata
+            else:
+                result.execution.metadata.append(key, value)
+
+        if report.capstdout and self._store_logs:
             log_file = File()
             log_file.set_file_data(report.capstdout)
             log_file.name = "stdout"
             log_file.mime_type = "txt"
             result.execution.append_log(log_file)
 
-        if report.capstderr:
+        if report.capstderr and self._store_logs:
             log_file = File()
             log_file.set_file_data(report.capstderr)
             log_file.name = "stderr"
@@ -166,13 +224,16 @@ class OpenTmiReport:
         # dut.serial_number = ''
         # result.append_dut(dut)
 
-        result.execution.profiling['suite'] = dict(duration=self._suite_time_delta)
-        result.execution.profiling['numtests'] = self._numtests
+        result.execution.profiling['suite'] = dict(
+            duration=self._suite_time_delta,
+            numtests=self._numtests
+        )
         result.execution.profiling['generated_at'] = self._generated.isoformat()
 
     def _upload_report(self, result: Result):
         try:
-            self._client.post_result(result)
+            data = self._client.post_result(result)
+            print(data)
             self._uploaded_success += 1
         except Exception:  # pylint: disable=broad-except
             self._uploaded_failed += 1
@@ -188,17 +249,24 @@ class OpenTmiReport:
 
         # pylint: disable=expression-not-assigned
         [print(result.data) for result in self.results]
+        [print(test) for test in self.tests]
 
         token = self.config.getoption("opentmi_token")
+        pool = ThreadPool(10)
+
         try:
             self._client.login_with_access_token(token)
-
-            num_cores = multiprocessing.cpu_count()
-            Parallel(n_jobs=num_cores, backend='threading')\
-                (delayed(self._upload_report)(result) for result in self.results)
+            pool.map(self._client.update_testcase, self.tests)
+            pool.map(self._upload_report, self.results)
+            pool.close()
+            pool.join()
         except Exception as error:  # pylint: disable=broad-except
             print(error)
 
+    @staticmethod
+    def _get_test_key(item):
+        return '_'.join(map(str, list(item.location)))
+
     # pytest hooks
 
     def pytest_runtest_logreport(self, report):
@@ -223,14 +291,14 @@ class OpenTmiReport:
             elif report.skipped:
                 self._append_skipped(report)
 
-    def pytest_collectreport(self, report):
+    def pytest_itemcollected(self, item):
         """
-        collect report hook
-        :param report: TestReport
+        Test item collected hook
+        :param item: Collection
         :return: None
         """
-        if report.failed:
-            self._append_failed(report)
+        key = OpenTmiReport._get_test_key(item)
+        self._items[key] = item
 
     # pylint: disable=unused-argument
     def pytest_sessionstart(self, session):
